{% extends 'base.html' %}


{% block content %}
    <h1>{% include 'push/_info_disable_title.html' %}</h1>
    
    {% include 'push/_info_disable_txt.html' %}
{% endblock %}


{% block title %}
    {% include 'push/_info_disable_title.html' %}
{% endblock %}


{% block add_meta %}
    {{ block.super }}
    <meta name="robots" content="noindex,nofollow" />
{% endblock %}
